import "CoreLibs/graphics"
import "CoreLibs/timer"
import "pixie"

local gfx <const> = playdate.graphics

class("Ball", {
         direction = {x=5, y=5},
}).extends(Pixie)

function Ball:init(width, height, color, x, y)
   local img = gfx.image.new(width, height, color)
   Ball.super.init(self, img, x, y)
   self.collisionResponse = gfx.sprite.kCollisionTypeBounce

   self:levelUpTimer()

   return self
end

function Ball:levelUpTimer()
   playdate.timer.performAfterDelay(25000,
      function(ball)
         if ball.direction.x > 0 then
            ball.direction.x += 1
         else
            ball.direction.x -= 1
         end
         if ball.direction.y > 0 then
            ball.direction.y += 1
         else
            ball.direction.y -= 1
         end

         ball:levelUpTimer()
      end,
      self)
end

function Ball:move()
   local collisions
   collisions = select(3, self:moveWithCollisions(self.x + self.direction.x, self.y + self.direction.y))

   for i=1, #collisions, 1 do
      self:changeDirection(collisions[i].normal)
   end
   
   return collisions
end

function Ball:changeDirection(normal)
   local dx,dy = normal:unpack()
   if dx ~= 0 then
      self.direction.x *= -1
   end

   if dy ~= 0 then
      self.direction.y *= -1
   end
end

function Ball:setDirection(x, y)
   x = x or self.direction.x
   y = y or self.direction.y
   self.direction.x = x
   self.direction.y = y
end

function Ball:deflect()
   self:setDirection(1)
end

function Ball:_normalizePosition()
   local changeVector = Ball.super._normalizePosition(self)
   if changeVector[1] ~= 0 then
      self.direction.x = changeVector[1]
   end
   if changeVector[2] ~= 0 then
      self.direction.y = changeVector[2]
   end
end
